@@ -4,7 +4,7 @@
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+# https://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -26,10 +26,14 @@ from twine import utils
 
 def skip_upload(response, skip_existing, package):
     filename = package.basefilename
-    # NOTE(sigmavirus24): Old PyPI returns the first message while Warehouse
-    # returns the latter. This papers over the differences.
-    msg_400 = ('A file named "{0}" already exists for'.format(filename),
-               'File already exists')
+    msg_400 = (
+        # Old PyPI message:
+        'A file named "{}" already exists for'.format(filename),
+        # Warehouse message:
+        'File already exists',
+        # Nexus Repository OSS message:
+        'Repository does not allow updating assets',
+    )
     msg_403 = 'Not enough permissions to overwrite artifact'
     # NOTE(sigmavirus24): PyPI presently returns a 400 status code with the
     # error message in the reason attribute. Other implementations return a
@@ -38,7 +42,7 @@ def skip_upload(response, skip_existing, package):
     #    True) AND
     # 2. a) The response status code is 409 OR
     # 2. b) The response status code is 400 AND it has a reason that matches
-    #       what we expect PyPI to return to us. OR
+    #       what we expect PyPI or Nexus OSS to return to us. OR
     # 2. c) The response status code is 403 AND the text matches what we
     #       expect Artifactory to return to us.
     return (skip_existing and (response.status_code == 409 or
@@ -51,21 +55,20 @@ def upload(upload_settings, dists):
     dists = _find_dists(dists)
 
     # Determine if the user has passed in pre-signed distributions
-    signatures = dict(
-        (os.path.basename(d), d) for d in dists if d.endswith(".asc")
-    )
+    signatures = {os.path.basename(d): d for d in dists if d.endswith(".asc")}
     uploads = [i for i in dists if not i.endswith(".asc")]
     upload_settings.check_repository_url()
     repository_url = upload_settings.repository_config['repository']
 
-    print("Uploading distributions to {0}".format(repository_url))
+    print("Uploading distributions to {}".format(repository_url))
 
     repository = upload_settings.create_repository()
+    uploaded_packages = []
 
     for filename in uploads:
         package = PackageFile.from_filename(filename, upload_settings.comment)
         skip_message = (
-            "  Skipping {0} because it appears to already exist".format(
+            "  Skipping {} because it appears to already exist".format(
                 package.basefilename)
         )
 
@@ -90,16 +93,25 @@ def upload(upload_settings, dists):
         # by PyPI should never happen in reality. This should catch malicious
         # redirects as well.
         if resp.is_redirect:
-            raise exceptions.RedirectDetected(
-                ('"{0}" attempted to redirect to "{1}" during upload.'
-                 ' Aborting...').format(repository_url,
-                                        resp.headers["location"]))
+            raise exceptions.RedirectDetected.from_args(
+                repository_url,
+                resp.headers["location"],
+            )
 
         if skip_upload(resp, upload_settings.skip_existing, package):
             print(skip_message)
             continue
+
         utils.check_status_code(resp, upload_settings.verbose)
 
+        uploaded_packages.append(package)
+
+    release_urls = repository.release_urls(uploaded_packages)
+    if release_urls:
+        print('\nView at:')
+        for url in release_urls:
+            print(url)
+
     # Bug 28. Try to silence a ResourceWarning by clearing the connection
     # pool.
     repository.close()
